@@ -44,20 +44,28 @@
 #include <assert.h>
 #include "color.h"
 
+#define DEFAULT_FRAME_RATE_INTERVAL 1 /* seconds */
+
 /* compositor <-> renderer interface */
 
-struct weston_renderbuffer {
-	pixman_region32_t damage;
-	int refcount;
+/** Opaque pointer to renderbuffer data.
+ */
+typedef void *weston_renderbuffer_t;
 
-	void (*destroy)(struct weston_renderbuffer *renderbuffer);
-};
-
-struct weston_renderbuffer *
-weston_renderbuffer_ref(struct weston_renderbuffer *renderbuffer);
-
-void
-weston_renderbuffer_unref(struct weston_renderbuffer *renderbuffer);
+/** Callback emitted when a renderbuffer is discarded
+ *
+ * \param renderbuffer The renderbuffer being discarded.
+ * \param user_data User data.
+ * \return true on success, false otherwise.
+ *
+ * A renderbuffer can be discarded by the renderer on various occasions, such as
+ * when the output is resized. Before signal emission, the renderer releases
+ * most allocated resources and marks it as stale. It is kept in the renderer's
+ * renderbuffer list until destruction with destroy_renderbuffer(), which can
+ * safely be called from the \c discarded callback.
+ */
+typedef bool (*weston_renderbuffer_discarded_func)(weston_renderbuffer_t renderbuffer,
+						   void *user_data);
 
 struct weston_renderer_options {
 };
@@ -68,6 +76,13 @@ struct linux_dmabuf_memory {
 	void (*destroy)(struct linux_dmabuf_memory *dmabuf);
 };
 
+enum weston_renderer_border_side {
+	WESTON_RENDERER_BORDER_TOP = 0,
+	WESTON_RENDERER_BORDER_LEFT = 1,
+	WESTON_RENDERER_BORDER_RIGHT = 2,
+	WESTON_RENDERER_BORDER_BOTTOM = 3,
+};
+
 struct weston_renderer {
 	int (*read_pixels)(struct weston_output *output,
 			   const struct pixel_format_info *format, void *pixels,
@@ -75,7 +90,7 @@ struct weston_renderer {
 			   uint32_t width, uint32_t height);
 	void (*repaint_output)(struct weston_output *output,
 			       pixman_region32_t *output_damage,
-			       struct weston_renderbuffer *renderbuffer);
+			       weston_renderbuffer_t renderbuffer);
 
 	/** See weston_renderer_resize_output()
 	 *
@@ -100,7 +115,7 @@ struct weston_renderer {
 			      struct linux_dmabuf_buffer *buffer);
 
 	const struct weston_drm_format_array *
-			(*get_supported_formats)(struct weston_compositor *ec);
+			(*get_supported_dmabuf_formats)(struct weston_compositor *ec);
 
 	bool (*fill_buffer_info)(struct weston_compositor *ec,
 				 struct weston_buffer *buffer);
@@ -108,41 +123,70 @@ struct weston_renderer {
 	void (*buffer_init)(struct weston_compositor *ec,
 			    struct weston_buffer *buffer);
 
-	/**
-	 * Add DMABUF as renderbuffer to the output
+	/** Create a renderbuffer
 	 *
-	 * \param output The output to add the DMABUF renderbuffer for.
-	 * \param dmabuf The description object of the DMABUF to import.
-	 * \return A weston_renderbuffer on success, NULL on failure.
+	 * \param output The output to render.
+	 * \param format The renderbuffer pixel format.
+	 * \param buffer The destination buffer, or \c NULL.
+	 * \param stride The destination \c buffer stride in bytes, or 0.
+	 * \param discarded_cb The callback emitted on a discarded event, or
+	 * NULL.
+	 * \param user_data User data passed to \c discarded_cb.
+	 * \return An opaque renderbuffer pointer, or \c NULL on failure.
 	 *
-	 * This function imports the DMABUF memory as renderbuffer and adds
-	 * it to the output. The returned weston_renderbuffer can be passed to
-	 * repaint_output() to render into the DMABUF.
+	 * This function creates a renderbuffer of the requested format. The
+	 * renderer can then use it to repaint the \c output into the specified
+	 * destination \c buffer, which must be the same size as the \c output
+	 * (including borders), or into an internal buffer if \c NULL.
 	 *
-	 * The ownership of the linux_dmabuf_memory is transferred to the
-	 * returned weston_renderbuffer. The linux_dmabuf_memory will be
-	 * destroyed automatically when the weston_renderbuffer is destroyed.
+	 * Backends should provide a \c discarded_cb callback in order to
+	 * properly handle renderbuffer lifetime.
+	 *
+	 * See repaint_output().
 	 */
-	struct weston_renderbuffer *
-			(*create_renderbuffer_dmabuf)(struct weston_output *output,
-						      struct linux_dmabuf_memory *dmabuf);
+	weston_renderbuffer_t
+	(*create_renderbuffer)(struct weston_output *output,
+			       const struct pixel_format_info *format,
+			       void *buffer,
+			       int stride,
+			       weston_renderbuffer_discarded_func discarded_cb,
+			       void *user_data);
 
-	/**
-	 * Remove the DAMBUF renderbuffer from the output
+	/** Create a renderbuffer from a DMABUF
 	 *
-	 * \param output The output to remove a DMABUF renderbuffer from.
-	 * \param renderbuffer The weston_renderbuffer that shall be removed
+	 * \param output The output to render.
+	 * \param dmabuf The destination DMABUF, ownership is transferred to the
+	 * renderbuffer.
+	 * \param discarded_cb The callback emitted on a discarded event, or
+	 * NULL.
+	 * \param user_data User data passed to \c discarded_cb.
+	 * \return An opaque renderbuffer pointer, or \c NULL on failure.
 	 *
-	 * This function removes the DMABUF renderbuffer from the output.
+	 * This function creates a renderbuffer from a DMABUF. The renderer can
+	 * then use it to repaint the output into the specified destination \c
+	 * dmabuf, which must be the same size as the \c output (including
+	 * borders).
 	 *
-	 * This allows the backend to signal the renderer that it will no longer
-	 * use the renderbuffer for rendering and the renderer may free the
-	 * resources of the renderbuffer.
+	 * Backends should provide a \c discarded_cb callback in order to
+	 * properly handle renderbuffer lifetime.
+	 *
+	 * See repaint_output().
 	 */
-	void (*remove_renderbuffer_dmabuf)(struct weston_output *output,
-					   struct weston_renderbuffer *renderbuffer);
+	weston_renderbuffer_t
+	(*create_renderbuffer_dmabuf)(struct weston_output *output,
+				      struct linux_dmabuf_memory *dmabuf,
+				      weston_renderbuffer_discarded_func discarded_cb,
+				      void *user_data);
 
-	/* Allocate a DMABUF that can be imported as renderbuffer
+	/** Destroy a renderbuffer
+	 *
+	 * \param renderbuffer The renderbuffer to destroy.
+	 *
+	 * This function destroys a \c renderbuffer.
+	 */
+	void (*destroy_renderbuffer)(weston_renderbuffer_t renderbuffer);
+
+	/** Allocate a DMABUF that can be imported as renderbuffer
 	 *
 	 * \param renderer The renderer that allocated the DMABUF
 	 * \param width The width of the allocated DMABUF
@@ -164,7 +208,41 @@ struct weston_renderer {
 
 	enum weston_renderer_type type;
 	const struct gl_renderer_interface *gl;
+	const struct vulkan_renderer_interface *vulkan;
 	const struct pixman_renderer_interface *pixman;
+
+	/* Sets the output border.
+	 *
+	 * The side specifies the side for which we are setting the border.
+	 * The width and height are the width and height of the border.
+	 * The tex_width patemeter specifies the width of the actual
+	 * texture; this may be larger than width if the data is not
+	 * tightly packed.
+	 *
+	 * The top and bottom textures will extend over the sides to the
+	 * full width of the bordered window.  The right and left edges,
+	 * however, will extend only to the top and bottom of the
+	 * compositor surface.  This is demonstrated by the picture below:
+	 *
+	 * +-----------------------+
+	 * |          TOP          |
+	 * +-+-------------------+-+
+	 * | |                   | |
+	 * |L|                   |R|
+	 * |E|                   |I|
+	 * |F|                   |G|
+	 * |T|                   |H|
+	 * | |                   |T|
+	 * | |                   | |
+	 * +-+-------------------+-+
+	 * |        BOTTOM         |
+	 * +-----------------------+
+	 */
+	void (*output_set_border)(struct weston_output *output,
+				  enum weston_renderer_border_side side,
+				  int32_t width, int32_t height,
+				  int32_t tex_width, unsigned char *data);
+
 };
 
 struct weston_tearing_control {
@@ -172,7 +250,7 @@ struct weston_tearing_control {
 	bool may_tear;
 };
 
-void
+bool
 weston_renderer_resize_output(struct weston_output *output,
 			      const struct weston_size *fb_size,
 			      const struct weston_geometry *area);
@@ -446,8 +524,6 @@ void
 weston_surface_to_buffer_region(struct weston_surface *surface,
 				pixman_region32_t *surface_region,
 				pixman_region32_t *buffer_region);
-void
-weston_surface_schedule_repaint(struct weston_surface *surface);
 
 /* weston_spring */
 
@@ -468,14 +544,13 @@ weston_view_is_opaque(struct weston_view *ev, pixman_region32_t *region);
 bool
 weston_view_has_valid_buffer(struct weston_view *ev);
 
-bool
-weston_view_matches_output_entirely(struct weston_view *ev,
-				    struct weston_output *output);
-
 bool
 weston_view_takes_input_at_point(struct weston_view *view,
 				 struct weston_coord_surface surf_pos);
 
+void
+weston_view_geometry_dirty_internal(struct weston_view *view);
+
 void
 weston_paint_node_move_to_plane(struct weston_paint_node *pnode,
 				struct weston_plane *plane);
@@ -563,15 +638,31 @@ weston_drm_format_get_modifiers(const struct weston_drm_format *format,
 void
 weston_compositor_destroy_touch_calibrator(struct weston_compositor *compositor);
 
-enum paint_node_status {
-	PAINT_NODE_CLEAN = 0,
-	PAINT_NODE_OUTPUT_DIRTY = 1 << 0,
-	PAINT_NODE_VIEW_DIRTY = 1 << 1,
-	PAINT_NODE_VISIBILITY_DIRTY = 1 << 2,
-	PAINT_NODE_PLANE_DIRTY = 1 << 3,
-	PAINT_NODE_CONTENT_DIRTY = 1 << 4,
-	PAINT_NODE_BUFFER_DIRTY = 1 << 5,
-	PAINT_NODE_ALL_DIRTY = (1 << 6) - 1,
+
+/**
+ * Reasons why placing a view on a plane failed. Needed by the dma-buf feedback.
+ */
+enum try_view_on_plane_failure_reasons {
+	FAILURE_REASONS_NONE = 0,
+	FAILURE_REASONS_FORCE_RENDERER = 1 << 0,
+	FAILURE_REASONS_FB_FORMAT_INCOMPATIBLE = 1 << 1,
+	FAILURE_REASONS_DMABUF_MODIFIER_INVALID = 1 << 2,
+	FAILURE_REASONS_ADD_FB_FAILED = 1 << 3,
+	FAILURE_REASONS_NO_PLANES_AVAILABLE = 1 << 4,
+	FAILURE_REASONS_PLANES_REJECTED = 1 << 5,
+	FAILURE_REASONS_INADEQUATE_CONTENT_PROTECTION = 1 << 6,
+	FAILURE_REASONS_INCOMPATIBLE_TRANSFORM = 1 << 7,
+	FAILURE_REASONS_NO_BUFFER = 1 << 8,
+	FAILURE_REASONS_BUFFER_TOO_BIG = 1 << 9,
+	FAILURE_REASONS_BUFFER_TYPE = 1 << 10,
+	FAILURE_REASONS_GLOBAL_ALPHA = 1 << 11,
+	FAILURE_REASONS_NO_GBM = 1 << 12,
+	FAILURE_REASONS_GBM_BO_IMPORT_FAILED = 1 << 13,
+	FAILURE_REASONS_GBM_BO_GET_HANDLE_FAILED = 1 << 14,
+	FAILURE_REASONS_NO_COLOR_TRANSFORM = 1 << 15,
+	FAILURE_REASONS_SOLID_SURFACE = 1 << 16,
+	FAILURE_REASONS_OCCLUDED_BY_RENDERER = 1 << 17,
+	FAILURE_REASONS_OUTPUT_COLOR_EFFECT = 1 << 18,
 };
 
 /**
@@ -596,7 +687,7 @@ struct weston_paint_node {
 
 	/* Mutable members: */
 
-	enum paint_node_status status;
+	enum weston_paint_node_status status;
 	struct weston_matrix buffer_to_output_matrix;
 	struct weston_matrix output_to_buffer_matrix;
 	bool needs_filtering;
@@ -607,7 +698,9 @@ struct weston_paint_node {
 	/* struct weston_output::paint_node_z_order_list */
 	struct wl_list z_order_link;
 
+	pixman_region32_t visible_previous;
 	pixman_region32_t visible;
+	pixman_region32_t clipped_view;
 	pixman_region32_t damage; /* In global coordinates */
 	struct weston_plane *plane;
 	struct weston_plane *plane_next;
@@ -618,9 +711,43 @@ struct weston_paint_node {
 	uint32_t try_view_on_plane_failure_reasons;
 	bool is_fully_opaque;
 	bool is_fully_blended;
-	bool is_direct;
+
+	/* This node's contents are solid, either from a solid buffer or a
+	 * placeholder.
+	 *
+	 * Care is taken to ensure that it is correct both during
+	 * assign_planes and during render, even if the value is different
+	 * for each during a single repaint.
+	 */
 	bool draw_solid;
+
+	/* This node's buffer or view alpha causes it to be completely
+	 * transparent, so it can be optimized away or skipped for
+	 * plane assignment.
+	 */
+	bool is_fully_transparent;
+
+	/* censored content must not be placed on a plane, it will be rendered
+	 * as a placeholder, and draw_solid must be set.
+	 */
+	bool censored;
+
+	/* Only valid when draw_solid is true, this is the solid color of
+	 * the paint node. It may simply be a copy of a solid buffer's
+	 * values, or it may be a placeholder used to replace a buffer's
+	 * content.
+	 *
+	 * It may change after assign_planes if plane assignment requires
+	 * the renderer to use a placeholder.
+	 */
 	struct weston_solid_buffer_values solid;
+
+	/* need_hole means this paint node has been placed on a plane beneath
+	 * the renderer's plane, so the renderer must draw a transparent hole
+	 * for the paint node.
+	 *
+	 * This is set in the backend assign_planes callback.
+	 */
 	bool need_hole;
 	uint32_t psf_flags; /* presentation-feedback flags */
 };
@@ -645,11 +772,53 @@ weston_surface_build_buffer_matrix(const struct weston_surface *surface,
 void
 weston_output_update_matrix(struct weston_output *output);
 
-void
+static inline void
 convert_size_by_transform_scale(int32_t *width_out, int32_t *height_out,
 				int32_t width, int32_t height,
 				uint32_t transform,
-				int32_t scale);
+				int32_t scale)
+{
+	assert(scale > 0);
+
+	switch (transform) {
+	case WL_OUTPUT_TRANSFORM_NORMAL:
+	case WL_OUTPUT_TRANSFORM_180:
+	case WL_OUTPUT_TRANSFORM_FLIPPED:
+	case WL_OUTPUT_TRANSFORM_FLIPPED_180:
+		*width_out = width / scale;
+		*height_out = height / scale;
+		break;
+	case WL_OUTPUT_TRANSFORM_90:
+	case WL_OUTPUT_TRANSFORM_270:
+	case WL_OUTPUT_TRANSFORM_FLIPPED_90:
+	case WL_OUTPUT_TRANSFORM_FLIPPED_270:
+		*width_out = height / scale;
+		*height_out = width / scale;
+		break;
+	default:
+		assert(0 && "invalid transform");
+	}
+}
+
+static inline bool
+convert_buffer_size_by_transform_scale(int32_t *width_out, int32_t *height_out,
+				       const struct weston_buffer *buf,
+				       const struct weston_buffer_viewport *vp)
+{
+	/* Buffer dimensions must be integer multiples of the scale */
+	if (buf->width % vp->buffer.scale ||
+	    buf->height % vp->buffer.scale)
+		return false;
+
+	convert_size_by_transform_scale(width_out, height_out,
+					buf->width, buf->height,
+					vp->buffer.transform,
+					vp->buffer.scale);
+	if (*width_out == 0 || *height_out == 0)
+		return false;
+
+	return true;
+}
 
 /* User authentication for remote backends */
 
@@ -660,4 +829,59 @@ void
 weston_output_copy_native_mode(struct weston_output *output,
 			       struct weston_mode *mode);
 
+static inline void
+region_init_infinite(pixman_region32_t *region)
+{
+	pixman_region32_init_rect(region, INT32_MIN, INT32_MIN,
+				  UINT32_MAX, UINT32_MAX);
+}
+
+struct weston_subsurface *
+weston_surface_to_subsurface(struct weston_surface *surface);
+
+void
+weston_presentation_feedback_discard_list(struct wl_list *list);
+
+void
+weston_surface_update_size(struct weston_surface *surface);
+
+/* Surface state helpers from surface-state.c */
+
+void
+weston_surface_commit(struct weston_surface *surface);
+
+void
+weston_subsurface_set_synchronized(struct weston_subsurface *sub, bool sync);
+
+void
+weston_surface_state_init(struct weston_surface *surface,
+			  struct weston_surface_state *state);
+
+void
+weston_surface_state_fini(struct weston_surface_state *state);
+
+const char *
+weston_plane_failure_reasons_to_str(enum try_view_on_plane_failure_reasons failure_reasons);
+
+uint32_t
+weston_surface_visibility_mask(struct weston_surface *surface);
+
+void
+weston_compositor_apply_transactions(struct weston_compositor *compositor);
+
+void
+weston_repaint_timer_arm(struct weston_compositor *compositor);
+
+struct timespec
+weston_output_repaint_from_present(const struct weston_output *output,
+				   const struct timespec *now,
+				   const struct timespec *present_time);
+
+void
+weston_backend_set_deferred(struct weston_backend *backend);
+
+void
+weston_backend_clear_deferred(struct weston_backend *backend,
+                              struct weston_compositor *compositor);
+
 #endif
